;; 
;;
PRO ABSTRACT_RESPIRATORY_SIGNAL_GATED_COMPARATOR__DEFINE

    ; define the class structure 
    struct = { ABSTRACT_RESPIRATORY_SIGNAL_GATED_COMPARATOR, INHERITS abstract_respiratory_signal_comparator, physiological_signal:obj_new(), gates_computer:obj_new(), gating_method:0, number_of_gates:0 }

END


;; 
;;
FUNCTION ABSTRACT_RESPIRATORY_SIGNAL_GATED_COMPARATOR::INIT, DURATION=DURATION, GATING_METHOD=GATING_METHOD, NUMBER_OF_GATES=NUMBER_OF_GATES

    if (~self->abstract_respiratory_signal_comparator::init(duration=duration)) then begin
        return, 0
    endif

    ; try to initialize field values
    catch, e
    if (e ne 0) then begin
        message, !error_state.msg, /informational
        return, 0
    endif
        ; try to initialize field values
        self->set_gating_method, gating_method
        self->set_number_of_gates, number_of_gates
    catch, /cancel

    ; setup the respiratory signal which will hold the data to gate
    self.physiological_signal = obj_new('physiologicalsignal')
    if (~obj_valid(self.physiological_signal)) then begin
        return, 0
    endif

    ; setup the gates computer which will perform gates calculations
    self.gates_computer = obj_new('gatescomputer')
    if (~obj_valid(self.gates_computer)) then begin
        return, 0
    endif
    self.gates_computer->setphysiologicalsignal, self.physiological_signal

    return, 1

END



;;
;;
PRO ABSTRACT_RESPIRATORY_SIGNAL_GATED_COMPARATOR::CLEANUP

    self->abstract_respiratory_signal_comparator::cleanup
    obj_destroy, self.gates_computer
    obj_destroy, self.physiological_signal

END


;;
;;
PRO ABSTRACT_RESPIRATORY_SIGNAL_GATED_COMPARATOR::SET_GATING_METHOD, METHOD

    ; reset to 1, if no value given
    if (n_elements(method) eq 0) then begin
        self.gating_method = 1
        return
    endif

    self.gating_method = method

END


;;
;;
FUNCTION ABSTRACT_RESPIRATORY_SIGNAL_GATED_COMPARATOR::GET_GATING_METHOD

    return, self.gating_method

END


;; 
;;
PRO ABSTRACT_RESPIRATORY_SIGNAL_GATED_COMPARATOR::SET_NUMBER_OF_GATES, NUMBER

    ; reset to 2, if no value given
    if (n_elements(number) eq 0) then begin
        self.number_of_gates = 2
        return
    endif

    self.number_of_gates = number

END


;;
;;
FUNCTION ABSTRACT_RESPIRATORY_SIGNAL_GATED_COMPARATOR::GET_NUMBER_OF_GATES

    return, self.number_of_gates

END


;;
;;
FUNCTION ABSTRACT_RESPIRATORY_SIGNAL_GATED_COMPARATOR::GATE, RESPIRATORY_SIGNALS, OFFSETS

    ; resample all signals and get the resulting sampling time
    signal_matrix = self->resample(respiratory_signals, offsets, /truncate, sampling_time=sampling_time)

    ; apply gating to all signals
    n_signals = n_elements(respiratory_signals)
    for i = 0, n_signals-1 do begin
        (self.physiological_signal)->setsignal, reform(signal_matrix[i,*]), sampling_time
        (self.gates_computer)->computegates, self.number_of_gates
        signal_matrix[i,*] = (self.gates_computer)->getgates(self.gating_method)
    endfor

    return, signal_matrix

END
