;;
;;
PRO CW_ABSTRACT_DRAW_MANIPULATOR__DEFINE

    struct = { CW_ABSTRACT_DRAW_MANIPULATOR, widget_id:0l }

END


;;
;;
FUNCTION CW_ABSTRACT_DRAW_MANIPULATOR::INIT, PARENT, UVALUE=UVALUE, UNAME=UNAME, TITLE=TITLE, MAP=MAP, SENSITIVE=SENSITIVE, $
         ALIGN_LEFT=ALIGN_LEFT, ALIGN_RIGHT=ALIGN_RIGHT, ALIGN_TOP=ALIGN_TOP, ALIGN_BOTTOM=ALIGN_BOTTOM, ALIGN_CENTER=ALIGN_CENTER, $
         UNITS=UNITS, FRAME=FRAME, XSIZE=XSIZE, YSIZE=YSIZE, XOFFSET=XOFFSET, YOFFSET=YOFFSET

    ; TODO check parameters (valid base, valid value, valid size, etc...)

    ; create the base widget of this draw manipulator and register its handlers
    self.widget_id = widget_base(parent, uvalue=uvalue, uname=uname, title=title, map=map, sensitive=sensitive, $
                                 /column, align_left=align_left, align_right=align_right, $ 
                                 align_top=align_top, align_bottom=align_bottom, align_center=align_center, $
                                 units=units, frame=frame, xsize=xsize, ysize=ysize, xoffset=xoffset, yoffset=yoffset, $
                                 func_get_value='cw_abstract_draw_manipulator_get_value', $
                                 pro_set_value='cw_abstract_draw_manipulator_set_value', $
                                 event_func='cw_abstract_draw_manipulator_event')

    ; create a draw area and a user interface area this draw manipulator comprises of
    ; the uvalue of the draw area is also used to store a reference to this stash object
    draw_area = widget_draw(self.widget_id, uname='draw_area', uvalue=self, $
                            /app_scroll, /expose_events, xsize=100, ysize=100, x_scroll_size=100, y_scroll_size=100, $
                            kill_notify='cw_abstract_draw_manipulator_kill_notify')
    ui_area = widget_base(self.widget_id, uname='ui_area', /column, frame=0, xpad=0, ypad=0)

    return, 1

END


;; Does nothing. 
;;
PRO CW_ABSTRACT_DRAW_MANIPULATOR::CLEANUP
END


;;
;;
FUNCTION CW_ABSTRACT_DRAW_MANIPULATOR::GET_WIDGET

    return, self.widget_id

END


;;
;;
FUNCTION CW_ABSTRACT_DRAW_MANIPULATOR::GET_DRAW_AREA

    return, widget_info(self.widget_id, find_by_uname='draw_area')

END


;;
;;
FUNCTION CW_ABSTRACT_DRAW_MANIPULATOR::GET_UI_AREA

    return, widget_info(self.widget_id, find_by_uname='ui_area')

END


;;
;;
FUNCTION CW_ABSTRACT_DRAW_MANIPULATOR::EVENT, EV

    if (ev.id eq self->get_widget()) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_base', /fold_case)) then begin
            self->set_size, ev.x, ev.y
            return, 0
        endif
    endif

    if (ev.id eq self->get_draw_area()) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_draw', /fold_case)) then begin
            if ((ev.type ge 2) and (ev.type le 4)) then begin
                self->redraw
                return, 0
            endif
        endif
    endif

    return, 0

END


;;
;;
FUNCTION CW_ABSTRACT_DRAW_MANIPULATOR::POST_EVENT, TOP, TYPE

    return, { DRAW_MANIPULATOR_EVENT, id:self->get_widget(), top:top, handler:0l, type:type }

END


;; TODO implement delayed execution of the set size method, scheduled by a timer
;;
PRO CW_ABSTRACT_DRAW_MANIPULATOR::SET_SIZE, X, Y
  
    self_geometry = widget_info(self->get_widget(), /geometry)
    ui_area_geometry = widget_info(self->get_ui_area(), /geometry)

    xsize = x - 2*self_geometry.xpad
    ysize = y - 2*self_geometry.ypad - self_geometry.space - ui_area_geometry.scr_ysize
    widget_control, self->get_draw_area(), xsize=xsize, ysize=ysize
    self->redraw

END


;;
;;
PRO CW_ABSTRACT_DRAW_MANIPULATOR::SET_DRAW_SIZE, X, Y

    ; limit the displayed size to the maximum values supported by the draw widget
    if (x gt 65535) then begin
        x = 65535
        message, 'Cannot set a draw size bigger than 65535!', /informational
    endif
    if (y gt 65535) then begin
        y = 65535
        message, 'Cannot set a draw size bigger than 65535!', /informational
    endif

    ; get information about old size and viewport
    draw_area = self->get_draw_area()
    draw_geometry = widget_info(draw_area, /geometry)
    widget_control, draw_area, get_draw_view=draw_position

    ; set new size and update viewport position
    widget_control, draw_area, draw_xsize=x, draw_ysize=y
    widget_control, draw_area, set_draw_view=[x*draw_position[0]/draw_geometry.draw_xsize, y*draw_position[1]/draw_geometry.draw_ysize]

END


;;
;;
FUNCTION CW_ABSTRACT_DRAW_MANIPULATOR::GET_DRAW_SIZE

    draw_geometry = widget_info(self->get_draw_area(), /geometry)
    return, [draw_geometry.draw_xsize, draw_geometry.draw_ysize]
END


;;
;;
PRO CW_ABSTRACT_DRAW_MANIPULATOR::REDRAW

    ; set output to the draw area window
    draw_area = self->get_draw_area()
    widget_control, draw_area, get_value=draw_area_window
    wset, draw_area_window
    erase

    ; if valid, plot the data according to the current viewport
    if (self->is_data_valid()) then begin
        widget_control, draw_area, get_draw_view=draw_position
        draw_geometry = widget_info(draw_area, /geometry)
        self->plot_data, draw_position[0], draw_position[1], draw_geometry.xsize, draw_geometry.ysize
    endif

END


;;
;;
FUNCTION CW_ABSTRACT_DRAW_MANIPULATOR::IS_DATA_VALID

    return, 0

END


;; Overwrite this method to actually plot something.
;;
PRO CW_ABSTRACT_DRAW_MANIPULATOR::PLOT_DATA, X, Y, XSIZE, YSIZE
END


;;
;;
FUNCTION CW_ABSTRACT_DRAW_MANIPULATOR_GET_VALUE, ID

    widget_control, widget_info(id, find_by_uname='draw_area'), get_uvalue=self
    return, self

END


;; does nothing 
;;
PRO CW_ABSTRACT_DRAW_MANIPULATOR_SET_VALUE, ID, VALUE
END


;;
;;
FUNCTION CW_ABSTRACT_DRAW_MANIPULATOR_EVENT, EV

    self = cw_abstract_draw_manipulator_get_value(ev.handler)
    return, self->event(ev)

END


;;
;;
PRO CW_ABSTRACT_DRAW_MANIPULATOR_KILL_NOTIFY, ID

    widget_control, id, get_uvalue=self
    obj_destroy, self

END
