;;
;;
PRO CW_ADVANCED_DRAW_MANIPULATOR__DEFINE

    struct = { CW_ADVANCED_DRAW_MANIPULATOR, INHERITS cw_abstract_draw_manipulator, sampling_time:0.0, xsize:0l, ysize:0l, xoffset:0.0, yoffset:0.0, xscale:0.0, yscale:0.0 }

END


;;
;;
FUNCTION CW_ADVANCED_DRAW_MANIPULATOR::INIT, PARENT, UVALUE=UVALUE, UNAME=UNAME, TITLE=TITLE, MAP=MAP, SENSITIVE=SENSITIVE, $
         ALIGN_LEFT=ALIGN_LEFT, ALIGN_RIGHT=ALIGN_RIGHT, ALIGN_TOP=ALIGN_TOP, ALIGN_BOTTOM=ALIGN_BOTTOM, ALIGN_CENTER=ALIGN_CENTER, $
         UNITS=UNITS, FRAME=FRAME, XSIZE=XSIZE, YSIZE=YSIZE, XOFFSET=XOFFSET, YOFFSET=YOFFSET

    ; use the constructor of the superclass to create an abstract draw manipulator widget
    if (~self->cw_abstract_draw_manipulator::init( $
            parent, uvalue=uvalue, uname=uname, title=title, map=map, sensitive=sensitive, $
            align_left=align_left, align_right=align_right, align_top=align_top, align_bottom=align_bottom, align_center=align_center, $
            units=units, frame=frame, xsize=xsize, ysize=ysize, xoffset=xoffset, yoffset=yoffset)) then begin
        return, 0
    endif

    ; initialize fields
    self.sampling_time = 1.0
    self.xsize = -1
    self.ysize = -1
    self.xoffset = 0
    self.yoffset = 0
    self.xscale = 1.0
    self.yscale = 1.0

    ; fill the user interface area
    ui_area = self->get_ui_area()

    transformation_form = widget_base(ui_area, uname='transformation_form', /row, frame=0, xpad=0, ypad=0)

    xoffset_label = widget_label(transformation_form, value='Shift Horizontally: ')
    xoffset_text = widget_text(transformation_form, uname='xoffset_text', value='0', xsize=8, /editable)
    xreset_button = widget_button(transformation_form, uname='xreset_button', value='Reset')

    yoffset_label = widget_label(transformation_form, value='Shift Vertically: ')
    yoffset_text = widget_text(transformation_form, uname='yoffset_text', value='0', xsize=8, /editable)
    yreset_button = widget_button(transformation_form, uname='yreset_button', value='Reset')

    xscale_label = widget_label(transformation_form, value='Scale Width: ')
    xscale_text = widget_text(transformation_form, uname='xscale_text', value='1.0', xsize=8, /editable)
    xfit_button = widget_button(transformation_form, uname='xfit_button', value='Fit')

    yscale_label = widget_label(transformation_form, value='Scale Height: ')
    yscale_text = widget_text(transformation_form, uname='yscale_text', value='1.0', xsize=8, /editable)
    yfit_button = widget_button(transformation_form, uname='yfit_button', value='Fit')

    return, 1

END


;;
;;
PRO CW_ADVANCED_DRAW_MANIPULATOR::CLEANUP

    self->cw_abstract_draw_manipulator::cleanup

END


;; 
;;
PRO CW_ADVANCED_DRAW_MANIPULATOR::SET_SAMPLING_TIME, SAMPLING_TIME

    self.sampling_time = sampling_time

END


;;
;;
FUNCTION CW_ADVANCED_DRAW_MANIPULATOR::GET_SAMPLING_TIME

    return, self.sampling_time

END


;; 
;;
PRO CW_ADVANCED_DRAW_MANIPULATOR::SET_DATA_SIZE, X, Y

    self.xsize = x
    self.ysize = y
    self->set_draw_size, self.xsize * self.xscale, self.ysize * self.yscale

END


;; 
;;
FUNCTION CW_ADVANCED_DRAW_MANIPULATOR::GET_DATA_SIZE

    return, [self.xsize, self.ysize]

END


;;
;;
PRO CW_ADVANCED_DRAW_MANIPULATOR::SET_XOFFSET, OFFSET

    self.xoffset = offset

    xoffset_text = widget_info(self->get_ui_area(), find_by_uname='xoffset_text')
    widget_control, xoffset_text, set_value=string(offset, format='(G0.0)')
    self->redraw

END


;;
;;
FUNCTION CW_ADVANCED_DRAW_MANIPULATOR::GET_XOFFSET

    return, self.xoffset

END


;;
;;
PRO CW_ADVANCED_DRAW_MANIPULATOR::SET_YOFFSET, OFFSET

    self.yoffset = offset

    yoffset_text = widget_info(self->get_ui_area(), find_by_uname='yoffset_text')
    widget_control, yoffset_text, set_value=string(offset, format='(G0.0)')
    self->redraw

END


;;
;;
FUNCTION CW_ADVANCED_DRAW_MANIPULATOR::GET_YOFFSET

    return, self.yoffset

END


;; TODO check that the given scale is a positive number
;;
PRO CW_ADVANCED_DRAW_MANIPULATOR::SET_XSCALE, SCALE

    self.xscale = scale
    self->set_draw_size, (self->get_data_size())[0] * self.xscale, (self->get_data_size())[1] * self.yscale

    xscale_text = widget_info(self->get_ui_area(), find_by_uname='xscale_text')
    widget_control, xscale_text, set_value=string(scale, format='(G0.0)')
    self->redraw

END


;;
;;
PRO CW_ADVANCED_DRAW_MANIPULATOR::FIT_XSCALE

    if (self->is_data_valid()) then begin
        draw_geometry = widget_info(self->get_draw_area(), /geometry)
        self->set_xscale, draw_geometry.xsize / (self->get_data_size())[0]
    endif

END


;;
;;
FUNCTION CW_ADVANCED_DRAW_MANIPULATOR::GET_XSCALE

    return, self.xscale

END


;; TODO check that the given scale is a positive number
;;
PRO CW_ADVANCED_DRAW_MANIPULATOR::SET_YSCALE, SCALE

    self.yscale = scale
    self->set_draw_size, (self->get_data_size())[0] * self.xscale, (self->get_data_size())[1] * self.yscale

    yscale_text = widget_info(self->get_ui_area(), find_by_uname='yscale_text')
    widget_control, yscale_text, set_value=string(scale, format='(G0.0)')
    self->redraw

END


;;
;;
PRO CW_ADVANCED_DRAW_MANIPULATOR::FIT_YSCALE

    if (self->is_data_valid()) then begin
        draw_geometry = widget_info(self->get_draw_area(), /geometry)
        self->set_yscale, draw_geometry.ysize / (self->get_data_size())[1]
    endif

END


;;
;;
FUNCTION CW_ADVANCED_DRAW_MANIPULATOR::GET_YSCALE

    return, self.yscale

END


;;
;;
FUNCTION CW_ADVANCED_DRAW_MANIPULATOR::EVENT, EV

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='xoffset_text')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_text_ch', /fold_case)) then begin
            widget_control, ev.id, get_value=value
            self->set_xoffset, value
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='yoffset_text')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_text_ch', /fold_case)) then begin
            widget_control, ev.id, get_value=value
            self->set_yoffset, value
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='xscale_text')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_text_ch', /fold_case)) then begin
            widget_control, ev.id, get_value=value
            self->set_xscale, value
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='yscale_text')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_text_ch', /fold_case)) then begin
            widget_control, ev.id, get_value=value
            self->set_yscale, value
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='xreset_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->set_xoffset, 0
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='yreset_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->set_yoffset, 0
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='xfit_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->fit_xscale
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='yfit_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->fit_yscale
            return, 0
        endif
    endif

    return, self->cw_abstract_draw_manipulator::event(ev)

END
