;;
;;
PRO CW_COMPARISON_TABLE__DEFINE

    struct = { CW_COMPARISON_TABLE, widget_id:0l }

END


;;
;;
FUNCTION CW_COMPARISON_TABLE::INIT, TABLE_DATA, TITLE=TITLE, GROUP_LEADER=GROUP_LEADER, MODAL=MODAL

    ; check table data
    if (size(table_data, /n_dimensions) ne 2) then begin
        message, 'A two-dimensional array of data must be given to display a comparison table!', /informational
        return, 0
    endif
    table_dimensions = size(table_data, /dimensions)
    column_labels = string(indgen(table_dimensions[0])+1)
    row_labels = string(indgen(table_dimensions[1])+1)

    ; check for a title
    if (n_elements(title) eq 0) then begin
        title = 'Comparison Table'
    endif

    ; create 
    self.widget_id = widget_base(title=title, group_leader=group_leader, modal=modal, $
                                 /column, /align_center, /tlb_kill_request_events, $
                                 func_get_value='cw_comparison_table_get_value', $
                                 pro_set_value='cw_comparison_table_set_value', $
                                 event_func='cw_comparison_table_event')

    ; create the table to display comparison data
    ; the uvalue of the table is also used to store a reference to this stash object
    table = widget_table(self.widget_id, uname='table', uvalue=self, value=table_data, $
                         column_labels=column_labels, row_labels=row_labels, $
                         kill_notify='cw_comparison_table_kill_notify')

    ok_button = widget_button(self.widget_id, uname='ok_button', value='OK')
    widget_control, self.widget_id, cancel_button=ok_button, default_button=ok_button

    widget_control, self.widget_id, /realize
    return, 1

END


;; Does nothing. 
;;
PRO CW_COMPARISON_TABLE::CLEANUP
END


;;
;;
FUNCTION CW_COMPARISON_TABLE::GET_WIDGET

    return, self.widget_id

END


;;
;;
FUNCTION CW_COMPARISON_TABLE::EVENT, EV

    if (ev.id eq self->get_widget()) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_kill_request', /fold_case)) then begin
            self->close
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='ok_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->close
            return, 0
        endif
    endif

    return, 0

END


;;
;;
PRO CW_COMPARISON_TABLE::CLOSE

    widget_control, self->get_widget(), /destroy

END


;;
;;
FUNCTION CW_COMPARISON_TABLE_GET_VALUE, ID

    widget_control, widget_info(id, find_by_uname='table'), get_uvalue=self
    return, self

END


;; does nothing 
;;
PRO CW_COMPARISON_TABLE_SET_VALUE, ID, VALUE
END


;;
;;
FUNCTION CW_COMPARISON_TABLE_EVENT, EV

    self = cw_comparison_table_get_value(ev.handler)
    return, self->event(ev)

END


;;
;;
PRO CW_COMPARISON_TABLE_KILL_NOTIFY, ID

    widget_control, id, get_uvalue=self
    obj_destroy, self

END
