;;
;;
PRO CW_MARKER_BASED_RESPIRATORY_SIGNAL__DEFINE

    struct = { CW_MARKER_BASED_RESPIRATORY_SIGNAL, INHERITS cw_respiratory_signal, wrapper_widget_id:0l, marker_processor:obj_new() }

END


;;
;;
FUNCTION CW_MARKER_BASED_RESPIRATORY_SIGNAL::INIT, PARENT, UVALUE=UVALUE, UNAME=UNAME, TITLE=TITLE, MAP=MAP, SENSITIVE=SENSITIVE, $
         ALIGN_LEFT=ALIGN_LEFT, ALIGN_RIGHT=ALIGN_RIGHT, ALIGN_TOP=ALIGN_TOP, ALIGN_BOTTOM=ALIGN_BOTTOM, ALIGN_CENTER=ALIGN_CENTER, $
         UNITS=UNITS, FRAME=FRAME, XSIZE=XSIZE, YSIZE=YSIZE, XOFFSET=XOFFSET, YOFFSET=YOFFSET

    
    ; create the base widget of this compound widget and register its handlers
    self.wrapper_widget_id = widget_base(parent, uvalue=uvalue, uname=uname, title=title, map=map, sensitive=sensitive, $
                                         /column, align_left=align_left, align_right=align_right, $ 
                                         align_top=align_top, align_bottom=align_bottom, align_center=align_center, $
                                         units=units, frame=frame, xsize=xsize, ysize=ysize, xoffset=xoffset, yoffset=yoffset, $
                                         func_get_value='cw_marker_based_respiratory_signal_get_value', $
                                         pro_set_value='cw_marker_based_respiratory_signal_set_value', $
                                         event_func='cw_marker_based_respiratory_signal_event')

    ; create the tab widget to show different aspects of marker data and respiration signal
    tabs_area = widget_tab(self.wrapper_widget_id, uname='wrapper_tabs_area')
    
    ; use the constructor of the superclass to create the page showing respiration curves
    ; the page is also used to store a reference to this stash object
    if (~self->cw_respiratory_signal::init(tabs_area, title='Respiratory Signal', uname='respiratory_signal_page')) then begin
        return, 0
    endif

    ; create a page for processing marker data
    self.marker_processor = obj_new('cw_marker_processor', tabs_area, title='ART Marker Data', uname='marker_processor_page')
    if (~obj_valid(self.marker_processor)) then begin
        return, 0
    endif

    ; fill the user interface area
    ui_area = widget_base(self.wrapper_widget_id, uname='wrapper_ui_area', /row)
    conversion_label = widget_label(ui_area, value='Convert marker data to respiration curve: ')
    conversion_method_droplist = widget_droplist(ui_area, uname='conversion_method_droplist', value=['Method: Digital Filter & PCA','Method: Digital Filter & Averaged Projection'])
    convert_button = widget_button(ui_area, value='Convert', uname='convert_button')
    
    return, 1

END


;;
;;
PRO CW_MARKER_BASED_RESPIRATORY_SIGNAL::CLEANUP

    self->cw_respiratory_signal::cleanup
    obj_destroy, self.marker_processor

END

;;
;;
FUNCTION CW_MARKER_BASED_RESPIRATORY_SIGNAL::GET_WRAPPER_WIDGET

    return, self.wrapper_widget_id

END


;;
;;
FUNCTION CW_MARKER_BASED_RESPIRATORY_SIGNAL::GET_WRAPPER_TABS_AREA

    return, widget_info(self.wrapper_widget_id, find_by_uname='wrapper_tabs_area')

END


;;
;;
FUNCTION CW_MARKER_BASED_RESPIRATORY_SIGNAL::GET_WRAPPER_UI_AREA

    return, widget_info(self.wrapper_widget_id, find_by_uname='wrapper_ui_area')

END


;;
;;
PRO CW_MARKER_BASED_RESPIRATORY_SIGNAL::SET_SIZE, X, Y

    ; calculate appropriate size for the tab pages
    self_geometry = widget_info(self->get_wrapper_widget(), /geometry)
    tabs_area_geometry = widget_info(self->get_wrapper_tabs_area(), /geometry)
    ui_area_geometry = widget_info(self->get_wrapper_ui_area(), /geometry)

    xsize = x - 2*self_geometry.xpad - (tabs_area_geometry.scr_xsize - tabs_area_geometry.xsize)
    ysize = y - 2*self_geometry.ypad - (tabs_area_geometry.scr_ysize - tabs_area_geometry.ysize) - self_geometry.space - ui_area_geometry.scr_ysize

    ; apply size to all tab pages
    self->cw_respiratory_signal::set_size, xsize, ysize
    (self.marker_processor)->set_size, xsize, ysize

    self->redraw

END


;;
;;
PRO CW_MARKER_BASED_RESPIRATORY_SIGNAL::CONVERT

    if ((self.marker_processor)->is_data_valid()) then begin

        ; get the user settings for the conversion method
        conversion_method_droplist = widget_info(self->get_wrapper_widget(), find_by_uname='conversion_method_droplist')
        method = widget_info(conversion_method_droplist, /droplist_select)

        ; call appropriate method to convert ART data 
        marker_data = (self.marker_processor)->get_marker_data()
        sampling_time = (self.marker_processor)->get_sampling_time()
        if (method eq 0) then begin
            signal_data = self->convert_by_digital_filter_and_pca(marker_data, sampling_time)
        endif else if (method eq 1) then begin
            signal_data = self->convert_by_digital_filter_and_averaged_projection(marker_data, sampling_time)
        endif else begin
            message, 'An unsupported method to convert marker data. This should never have happened!', /informational
        endelse

        ; create respiratory signal object
        ; TODO mention ART Marker Processor parameters and conversion method in the comment field
        respiratory_signal = obj_new('respiratorysignal')
        respiratory_signal->setsignal, signal_data, sampling_time, 'ART Camera'
    
        ; move to respiratory signal page and set the signal
        widget_control, self->get_wrapper_tabs_area(), set_tab_current=0
        self->set_respiratory_signal, respiratory_signal

        ; update file name display
        file_text = widget_info(self->get_ui_area(), find_by_uname='file_text')
        widget_control, file_text, set_value=''

    endif

END


;;
;;
FUNCTION CW_MARKER_BASED_RESPIRATORY_SIGNAL::CONVERT_BY_DIGITAL_FILTER_AND_PCA, MARKER_DATA, SAMPLING_TIME

    body_motion_digital_filter = obj_new('body_motion_digital_filter', sampling_time=sampling_time, multiplex_dimension=1)
    body_motion_data = body_motion_digital_filter->multiplex_apply(marker_data)
    obj_destroy, body_motion_digital_filter

    respiratory_motion_data = marker_data - body_motion_data
    respiratory_motion_components = calculate_art_pca(respiratory_motion_data)
    return, reform(respiratory_motion_components[0,*])

END


;;
;;
FUNCTION CW_MARKER_BASED_RESPIRATORY_SIGNAL::CONVERT_BY_DIGITAL_FILTER_AND_AVERAGED_PROJECTION, MARKER_DATA, SAMPLING_TIME

    body_motion_digital_filter = obj_new('body_motion_digital_filter', sampling_time=sampling_time, multiplex_dimension=1)
    body_motion_data = body_motion_digital_filter->multiplex_apply(marker_data)
    obj_destroy, body_motion_digital_filter

    respiratory_motion_data = marker_data - body_motion_data
    respiratory_motion_average = average_marker_data(respiratory_motion_data)
    return, reform(respiratory_motion_average[*,1])

END



;;
;;
PRO CW_MARKER_BASED_RESPIRATORY_SIGNAL::REDRAW

    current_tab = widget_info(self->get_wrapper_tabs_area(), /tab_current)
    if (current_tab eq 0) then begin
        self->cw_respiratory_signal::redraw
    endif else if (current_tab eq 0) then begin
        (self.marker_processor)->redraw
    endif

END


;;
;;
FUNCTION CW_MARKER_BASED_RESPIRATORY_SIGNAL::EVENT, EV

    ; use the superclass to handle events for the respiratory signal page
    if (ev.handler eq self->get_widget()) then begin
        return, self->cw_respiratory_signal::event(ev)
    end

    if (ev.id eq self->get_wrapper_widget()) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_base', /fold_case)) then begin
            self->set_size, ev.x, ev.y
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_wrapper_widget(), find_by_uname='convert_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->convert
            return, 0
        endif
    endif

    ; pass through draw manipulator events posted by child components
    if ((ev.id eq self->get_widget()) || (ev.id eq (self.marker_processor)->get_widget())) then begin
        if (strcmp(tag_names(ev, /structure), 'draw_manipulator_event', /fold_case)) then begin
            ev.id = self->get_wrapper_widget()
            ev.handler = 0
            return, ev
        endif
    endif

    return, 0

END


;;
;;
FUNCTION CW_MARKER_BASED_RESPIRATORY_SIGNAL_GET_VALUE, ID

    widget_control, widget_info(id, find_by_uname='respiratory_signal_page'), get_value=self
    return, self

END


;; does nothing 
;;
PRO CW_MARKER_BASED_RESPIRATORY_SIGNAL_SET_VALUE, ID, VALUE
END


;;
;;
FUNCTION CW_MARKER_BASED_RESPIRATORY_SIGNAL_EVENT, EV

    self = cw_marker_based_respiratory_signal_get_value(ev.handler)
    return, self->event(ev)

END
