;;
;;
PRO CW_MARKER_PROCESSOR__DEFINE

    struct = { CW_MARKER_PROCESSOR, INHERITS cw_advanced_draw_manipulator, marker_processor:obj_new() }

END


;;
;;
FUNCTION CW_MARKER_PROCESSOR::INIT, PARENT, UVALUE=UVALUE, UNAME=UNAME, TITLE=TITLE, MAP=MAP, SENSITIVE=SENSITIVE, $
         ALIGN_LEFT=ALIGN_LEFT, ALIGN_RIGHT=ALIGN_RIGHT, ALIGN_TOP=ALIGN_TOP, ALIGN_BOTTOM=ALIGN_BOTTOM, ALIGN_CENTER=ALIGN_CENTER, $
         UNITS=UNITS, FRAME=FRAME, XSIZE=XSIZE, YSIZE=YSIZE, XOFFSET=XOFFSET, YOFFSET=YOFFSET

    ; use the constructor of the superclass to create an advanced draw manipulator widget
    if (~self->cw_advanced_draw_manipulator::init( $
            parent, uvalue=uvalue, uname=uname, title=title, map=map, sensitive=sensitive, $
            align_left=align_left, align_right=align_right, align_top=align_top, align_bottom=align_bottom, align_center=align_center, $
            units=units, frame=frame, xsize=xsize, ysize=ysize, xoffset=xoffset, yoffset=yoffset )) then begin
        return, 0
    endif

    ; initialize the underlying ART processor
    self.marker_processor = obj_new('cached_art_calibrating_marker_processor', distance_threshold=10, reference_body_id=0)

    ; fill the user interface area
    ui_area = self->get_ui_area()

    marker_status = widget_base(ui_area, uname='marker_status', /row, frame=1, xpad=0, ypad=0)
    sampling_time_label = widget_label(marker_status, uname='sampling_time_label', value='Sampling Time: ', /dynamic_resize)
    sample_count_label = widget_label(marker_status, uname='sample_count_label', value='Sample Count: ', /dynamic_resize)
    duration_label = widget_label(marker_status, uname='duration_label', value='Overall Duration: ', /dynamic_resize)

    marker_form1 = widget_base(ui_area, uname='marker_form1', /row, frame=0, xpad=0, ypad=0, space=10)
    marker_form2 = widget_base(marker_form1, uname='marker_form2', /column, frame=0, xpad=0, ypad=0)

    marker_form3 = widget_base(marker_form2, uname='marker_form3', /row, frame=0, xpad=0, ypad=0)
    load_button = widget_button(marker_form3, value='Load from File...', uname='load_button')
    save_button = widget_button(marker_form3, value='Save to File...', uname='save_button')

    marker_form5 = widget_base(marker_form2, uname='marker_form5', /row, frame=0, xpad=0, ypad=0)
    distance_threshold_label = widget_label(marker_form5, uname='distance_threshold_label', value='Marker Movement Threshold:')
    distance_threshold_text = widget_text(marker_form5, uname='distance_threshold_text', value='10', xsize=5, /editable)
    
    marker_form6 = widget_base(marker_form2, uname='marker_form6', /row, frame=0, xpad=0, ypad=0)
    reference_body_label = widget_label(marker_form6, uname='reference_body_label', value='Reference Body:')
    distance_threshold_text = widget_combobox(marker_form6, uname='reference_body_combobox', value=string(indgen(30)), /editable)

    marker_form7 = widget_base(marker_form1, uname='marker_form7', /row, frame=1, xpad=0, ypad=0)
    marker_form8 = widget_base(marker_form6, uname='marker_form8', /column, /base_align_left, frame=0, xpad=0, ypad=0)
    relevant_markers_label = widget_label(marker_form8, uname='relevant_markers_label', value='Relevant Markers:')
    marker_form9 = widget_base(marker_form7, uname='marker_form9', /row, frame=0, xpad=0, ypad=0)
    find_relevant_markers_label = widget_label(marker_form9, uname='find_relevant_markers_label', value='Find by ')
    find_relevant_markers_text = widget_text(marker_form9, uname='find_relevant_markers_text', value='300', xsize=5, /editable)
    find_relevant_markers_droplist = widget_droplist(marker_form9, uname='find_relevant_markers_droplist', value=['seconds', 'samples', 'fraction'])
    find_relevant_markers_button = widget_button(marker_form9, value='>>', uname='find_relevant_markers_button')
    relevant_markers_list = widget_list(marker_form7, uname='relevant_markers_list', value='', xsize=3, ysize=8, /multiple)

    marker_form10 = widget_base(marker_form1, uname='marker_form10', /column, /base_align_left, frame=1, xpad=0, ypad=0)
    calibration_matrix_label = widget_label(marker_form10, uname='calibration_matrix_label', value='Calibration Matrix:')
    calibration_matrix_table = widget_table(marker_form10, uname='calibration_matrix_table', value=[[1.0,0.0,0.0,0.0],[0.0,1.0,0.0,0.0],[0.0,0.0,1.0,0.0],[0.0,0.0,0.0,1.0]], xsize=4, ysize=4, alignment=1, column_widths=50, /no_headers, /editable)
    marker_form11 = widget_base(marker_form10, uname='marker_form11', /row, /base_align_right, /align_right, frame=0, xpad=0, ypad=0)
    clear_calibration_matrix_button = widget_button(marker_form11, value='Clear', uname='clear_calibration_matrix_button')
    load_calibration_matrix_button = widget_button(marker_form11, value='Load...', uname='load_calibration_matrix_button')
    save_calibration_matrix_button = widget_button(marker_form11, value='Save...', uname='save_calibration_matrix_button')

    return, 1

END


;;
;;
PRO CW_MARKER_PROCESSOR::CLEANUP

    self->cw_advanced_draw_manipulator::cleanup

    ; delete art camera data, if necessary
    art_data = (self.marker_processor)->get_art_data()
    if (obj_valid(art_data)) then begin
        obj_destroy, art_data
    endif

    ; delete marker processor
    obj_destroy, self.marker_processor

END


;;
;;
FUNCTION CW_MARKER_PROCESSOR::IS_DATA_VALID

    if (~obj_valid(self.marker_processor)) then begin
        return, 0
    endif 
    if (~((self.marker_processor)->is_processible())) then begin
        return, 0
    endif
    return, 1

END


;;
;;
FUNCTION CW_MARKER_PROCESSOR::GET_MARKER_DATA

    if (self->is_data_valid()) then begin
        return, (self.marker_processor)->retrieve_markers()
    endif
    return, reform([-1], 1, 1, 1)

END


;; TODO later consider the zoom level here
;;
FUNCTION CW_MARKER_PROCESSOR::GET_MARKER_DATA_XSIZE

    if (self->is_data_valid()) then begin
        return, ((self.marker_processor)->get_art_data())->get_sample_count()
    endif
    return, -1

END


;;
;;
FUNCTION CW_MARKER_PROCESSOR::GET_MARKER_DATA_YSIZE

    if (self->is_data_valid()) then begin
        min = min((self.marker_processor)->retrieve_markers(), max=max)
        return, max - min
    endif
    return, -1

END


;;
;;
PRO CW_MARKER_PROCESSOR::LOAD_ART_CAMERA_DATA

    ; re-initialize underlying ART processor, if necessary
    if (~obj_valid(self.marker_processor)) then begin
        message, 'The ART processor somehow got corrupted. This should never have happened! Anyway, creating a new one...', /informational
        self.marker_processor = obj_new('cached_art_calibrating_marker_processor', distance_threshold=10, reference_body_id=0)
    endif

    ; delete old art camera data, if necessary
    ; TODO show confirmation dialog before deleting existing data?
    art_data = (self.marker_processor)->get_art_data()
    if (obj_valid(art_data)) then begin
        obj_destroy, art_data
    endif

    ; load marker data from a file
    art_data = obj_new('cached_art_camera_data')
    (self.marker_processor)->set_art_data, art_data

    ; reset relevant markers and data size and redraw markers
    self->reset_relevant_markers
    self->set_sampling_time, art_data->get_sampling_time()
    self->set_data_size, self->get_marker_data_xsize(), self->get_marker_data_ysize()
    self->redraw

END


;;
;;
PRO CW_MARKER_PROCESSOR::SAVE_ART_CAMERA_DATA

    ; check for valid marker data
    if (~self->is_data_valid()) then begin
        message, 'Could not save ART camera data, since none are loaded currently!', /informational
        return
    endif

    ; save marker data to file
    ((self.marker_processor)->get_art_data())->save_sav

END

;;
;;
PRO CW_MARKER_PROCESSOR::CHECK_NORMALIZE_WIDTH, SELECT

    self.normalize_width = select
    self->redraw

END


;;
;;
PRO CW_MARKER_PROCESSOR::CHECK_NORMALIZE_HEIGHT, SELECT

    self.normalize_height = select
    self->redraw

END


;;
;;
PRO CW_MARKER_PROCESSOR::SET_DISTANCE_THRESHOLD, VALUE

    ; try to set distance threshold
    catch, e
    if (e ne 0) then begin

        ; reset to old distance threshold on error
        message, 'The string ' + string(value) + ' is not a valid distance threshold. Resetting to old value...' , /informational
        distance_threshold_text = widget_info(self->get_widget(), find_by_uname='distance_threshold_text')
        widget_control, distance_threshold_text, set_value=(self.marker_processor)->get_distance_threshold()

    return
    endif

        ; try to set distance threshold
        (self.marker_processor)->set_distance_threshold, value

    catch, /cancel

    self->reset_relevant_markers
    self->set_data_size, self->get_marker_data_xsize(), self->get_marker_data_ysize()
    self->redraw

END


;;
;;
PRO CW_MARKER_PROCESSOR::SET_REFERENCE_BODY, VALUE

    ; try to set reference body
    catch, e
    if (e ne 0) then begin

        ; reset to old reference body on error
        message, 'The string ' + int2str(value) + ' is not a valid reference body id. Resetting to old value...' , /informational
        ; TODO implement

    return
    endif

        ; try to set reference body
        (self.marker_processor)->set_reference_body_id, value

    catch, /cancel

    self->set_data_size, self->get_marker_data_xsize(), self->get_marker_data_ysize()
    self->redraw

END


;;
;;
PRO CW_MARKER_PROCESSOR::SET_CALIBRATION_MATRIX, VALUE

    ; try to set calibration matrix
    catch, e
    if (e ne 0) then begin

        ; reset to old calibration matrix on error
        message, 'The value ' + string(value) + ' is not a valid 4x4 homogenous calibration matrix. Resetting to old value...' , /informational
        ; TODO implement

    return
    endif

        ; try to set calibration matrix
        (self.marker_processor)->set_calibration_matrix, value

    catch, /cancel

    self->set_data_size, self->get_marker_data_xsize(), self->get_marker_data_ysize()
    self->redraw

END


;;
;;
PRO CW_MARKER_PROCESSOR::CLEAR_CALIBRATION_MATRIX

    ; clear calibration matrix by setting it to an identity matrix
    (self.marker_processor)->set_calibration_matrix

    ; update table view and draw
    calibration_matrix = (self.marker_processor)->get_calibration_matrix()
    calibration_matrix_table = widget_info(self->get_widget(), find_by_uname='calibration_matrix_table')
    widget_control, calibration_matrix_table, set_value=calibration_matrix

    self->set_data_size, self->get_marker_data_xsize(), self->get_marker_data_ysize()
    self->redraw

END


;;
;;
PRO CW_MARKER_PROCESSOR::LOAD_CALIBRATION_MATRIX

    ; load calibration matrix from file
    (self.marker_processor)->load_calibration_matrix

    ; update table view and draw
    calibration_matrix = (self.marker_processor)->get_calibration_matrix()
    calibration_matrix_table = widget_info(self->get_widget(), find_by_uname='calibration_matrix_table')
    widget_control, calibration_matrix_table, set_value=calibration_matrix

    self->set_data_size, self->get_marker_data_xsize(), self->get_marker_data_ysize()
    self->redraw

END


;;
;;
PRO CW_MARKER_PROCESSOR::SAVE_CALIBRATION_MATRIX

    ; save calibration matrix to file
    (self.marker_processor)->save_calibration_matrix

END


;;
;;
PRO CW_MARKER_PROCESSOR::SET_RELEVANT_MARKERS, VALUES

    ; try to set relevant markers
    catch, e
    if (e ne 0) then begin

        ; reset to old relevant markers on error
        message, 'The not all of the values ' + int2str(value) + ' are valid marker slots. Resetting to old values...' , /informational
        ; TODO implement

    return
    endif

        ; try to set calibration body
        (self.marker_processor)->set_relevant_markers, values

    catch, /cancel

    self->set_data_size, self->get_marker_data_xsize(), self->get_marker_data_ysize()
    self->redraw

END


;;
;;
PRO CW_MARKER_PROCESSOR::FIND_RELEVANT_MARKERS

    if (self->is_data_valid()) then begin

        ; get the user settings for finding relevant markers
        find_relevant_markers_text = widget_info(self->get_widget(), find_by_uname='find_relevant_markers_text')
        widget_control, find_relevant_markers_text, get_value=value
        value = float(value[0])
        find_relevant_markers_droplist = widget_info(self->get_widget(), find_by_uname='find_relevant_markers_droplist')
        method = widget_info(find_relevant_markers_droplist, /droplist_select)

        ; call appropriate method to find relevant markers
        if (method eq 0) then begin
            (self.marker_processor)->find_relevant_markers, min_sample_duration=value
        endif else if (method eq 1) then begin
            (self.marker_processor)->find_relevant_markers, min_sample_count=value
        endif else if (method eq 2) then begin
            (self.marker_processor)->find_relevant_markers, min_sample_fraction=value
        endif else begin
            message, 'An unsupported method to find relevant markers was selected. This should never have happened!', /informational
        endelse

        ; update list view and draw
        relevant_markers = (self.marker_processor)->get_relevant_markers()
        relevant_markers_list = widget_info(self->get_widget(), find_by_uname='relevant_markers_list')
        widget_control, relevant_markers_list, set_list_select=relevant_markers

        self->set_data_size, self->get_marker_data_xsize(), self->get_marker_data_ysize()
        self->redraw

    endif

END


;;
;;
PRO CW_MARKER_PROCESSOR::RESET_RELEVANT_MARKERS

    ; invalidate list of relevant markers
    (self.marker_processor)->set_relevant_markers

    ; update list view
    relevant_markers_list = widget_info(self->get_widget(), find_by_uname='relevant_markers_list')
    if (self->is_data_valid()) then begin
        marker_count = (self.marker_processor)->retrieve_marker_count()
        widget_control, relevant_markers_list, set_value=int2str(indgen(marker_count))
    endif else begin
        widget_control, relevant_markers_list, set_value=''
    endelse

END


;;
;;
PRO CW_MARKER_PROCESSOR::PLOT_DATA, X, Y, XSIZE, YSIZE

    ; retrieve marker data and draw them
    xsize <= self->get_marker_data_xsize() * self.xscale
    ysize <= self->get_marker_data_ysize() * self.yscale
    markers = (self.marker_processor)->retrieve_markers()
    ymin = min(markers) + y/self.yscale + self.yoffset
    ymax = ymin + ysize/self.yscale
    markers_x = (findgen(xsize/self.xscale) + x/self.xscale) * self.sampling_time + self.xoffset
    markers_y = markers[(x/self.xscale):((x+xsize-1)/self.xscale),*,*] + self.yoffset

    ; establish coordinate system
    plot, markers_x, markers_x*0, yrange=[ymin,ymax], xstyle=1, ystyle=1, xticklen=0.1, yticklen=0.1, /nodata

    ; plot each dimension of each marker as a single curve
    for i = 0, (size(markers_y, /dimensions))[1]-1 do begin
        oplot, markers_x, markers_y[*,i,0], color=255L
        oplot, markers_x, markers_y[*,i,1], color=255L*256L
        oplot, markers_x, markers_y[*,i,2], color=255L*256L*256L
    end

END


;;
;;
FUNCTION CW_MARKER_PROCESSOR::EVENT, EV

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='load_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->load_art_camera_data
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='save_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->save_art_camera_data
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='distance_threshold_text')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_text_ch', /fold_case)) then begin
            widget_control, ev.id, get_value=value
            self->set_distance_threshold, value
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='reference_body_combobox')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_combobox', /fold_case)) then begin
            self->set_reference_body, ev.str
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='reference_body_combobox')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_combobox', /fold_case)) then begin
            self->set_reference_body, ev.str
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='relevant_markers_list')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_list', /fold_case)) then begin
            self->set_relevant_markers, widget_info(ev.id, /list_select)
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='find_relevant_markers_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->find_relevant_markers
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='calibration_matrix_table')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_table_ch', /fold_case)) then begin
            widget_control, ev.id, get_value=value
            self->set_calibration_matrix, value
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='clear_calibration_matrix_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->clear_calibration_matrix
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='load_calibration_matrix_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->load_calibration_matrix
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='save_calibration_matrix_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->save_calibration_matrix
            return, 0
        endif
    endif

    return, self->cw_advanced_draw_manipulator::event(ev)

END
