;;
;;
PRO CW_RESPIRATORY_SIGNAL__DEFINE

    struct = { CW_RESPIRATORY_SIGNAL, INHERITS cw_advanced_draw_manipulator, respiratory_signal:obj_new() }

END


;;
;;
FUNCTION CW_RESPIRATORY_SIGNAL::INIT, PARENT, UVALUE=UVALUE, UNAME=UNAME, TITLE=TITLE, MAP=MAP, SENSITIVE=SENSITIVE, $
         ALIGN_LEFT=ALIGN_LEFT, ALIGN_RIGHT=ALIGN_RIGHT, ALIGN_TOP=ALIGN_TOP, ALIGN_BOTTOM=ALIGN_BOTTOM, ALIGN_CENTER=ALIGN_CENTER, $
         UNITS=UNITS, FRAME=FRAME, XSIZE=XSIZE, YSIZE=YSIZE, XOFFSET=XOFFSET, YOFFSET=YOFFSET

    ; use the constructor of the superclass to create an advanced draw manipulator widget
    if (~self->cw_advanced_draw_manipulator::init( $
            parent, uvalue=uvalue, uname=uname, title=title, map=map, sensitive=sensitive, $
            align_left=align_left, align_right=align_right, align_top=align_top, align_bottom=align_bottom, align_center=align_center, $
            units=units, frame=frame, xsize=xsize, ysize=ysize, xoffset=xoffset, yoffset=yoffset )) then begin
        return, 0
    endif

    ; fill the user interface area
    ui_area = self->get_ui_area()

    signal_status = widget_base(ui_area, uname='signal_status', /row, frame=1, xpad=0, ypad=0)
    sampling_time_label = widget_label(signal_status, value='Sampling Time [s]: ')
    sampling_time_text = widget_text(signal_status, uname='sampling_time_text', value='', xsize=8)
    sample_count_label = widget_label(signal_status, value='Sample Count: ')
    sample_count_text = widget_text(signal_status, uname='sample_count_text', value='', xsize=8)
    duration_label = widget_label(signal_status, value='Overall Duration [s]: ')
    duration_text = widget_text(signal_status, uname='duration_text', value='', xsize=8)
    sensor_label = widget_label(signal_status, value='Sensor: ')
    sensor_text = widget_text(signal_status, uname='sensor_text', value='', xsize=16)
    comments_label = widget_label(signal_status, value='Comments: ')
    comments_text = widget_text(signal_status, uname='comments_text', value='', xsize=32)
    file_label = widget_label(signal_status, value='File (last loaded or saved): ')
    file_text = widget_text(signal_status, uname='file_text', value='', xsize=24)

    signal_actions = widget_base(ui_area, uname='signal_actions', /row, frame=1, xpad=0, ypad=0)
    load_button = widget_button(signal_actions, value='Load from File...', uname='load_button')
    save_button = widget_button(signal_actions, value='Save to File...', uname='save_button')
    invert_button = widget_button(signal_actions, value='Invert', uname='invert_button')
    syncronize_button = widget_button(signal_actions, uname='syncronize_button', value='Syncronize')

    return, 1

END


;;
;;
PRO CW_RESPIRATORY_SIGNAL::CLEANUP

    self->cw_advanced_draw_manipulator::cleanup

    ; delete signal data, if necessary
    if (obj_valid(self.respiratory_signal)) then begin
        obj_destroy, self.respiratory_signal
    endif

END


;;
;;
FUNCTION CW_RESPIRATORY_SIGNAL::IS_DATA_VALID

    if (~obj_valid(self.respiratory_signal)) then begin
        return, 0
    endif 
    if ((self.respiratory_signal)->getsize() le 0) then begin
        return, 0
    endif
    return, 1

END


;;
;;
FUNCTION CW_RESPIRATORY_SIGNAL::GET_SIGNAL

    if (self->is_data_valid()) then begin
        return, (self.respiratory_signal)->getsignal()
    endif
    return, -1

END


;;
;;
FUNCTION CW_RESPIRATORY_SIGNAL::GET_RESPIRATORY_SIGNAL

    if (self->is_data_valid()) then begin
        return, self.respiratory_signal
    endif
    return, obj_new()

END


;;
;;
PRO CW_RESPIRATORY_SIGNAL::SET_RESPIRATORY_SIGNAL, RESPIRATORY_SIGNAL

    ; check signal
    if (~obj_isa(respiratory_signal, 'respiratorysignal')) then begin
        message, 'The respiratory signal must be of type RESPIRATORYSIGNAL!' 
    endif

    ; delete old signal data, if necessary
    ; TODO show confirmation dialog before deleting existing data?
    if (obj_valid(self.respiratory_signal)) then begin
        obj_destroy, self.respiratory_signal
    endif

    ; set new signal
    self.respiratory_signal = respiratory_signal

    ; reset draw geometry
    self->set_sampling_time, (self.respiratory_signal)->getsamplingtime()
    xsize = (self.respiratory_signal)->getsize()
    min = min((self.respiratory_signal)->getsignal(), max=max, /nan)
    ysize = max - min
    self->set_data_size, xsize, ysize
    widget_control, self->get_draw_area(), set_draw_view=[0,0]

    ; update metadata display
    ui_area = self->get_ui_area()
    sampling_time_text = widget_info(ui_area, find_by_uname='sampling_time_text')
    widget_control, sampling_time_text, set_value=string(respiratory_signal->getsamplingtime(), format='(G0.0)')
    sample_count_text = widget_info(ui_area, find_by_uname='sample_count_text')
    widget_control, sample_count_text, set_value=string(respiratory_signal->getsize(), format='(I0)')
    duration_text = widget_info(ui_area, find_by_uname='duration_text')
    widget_control, duration_text, set_value=string(respiratory_signal->getduration(), format='(G0.0)')
    sensor_text = widget_info(ui_area, find_by_uname='sensor_text')
    widget_control, sensor_text, set_value=string(respiratory_signal->getsensor())
    comments_text = widget_info(ui_area, find_by_uname='comments_text')
    widget_control, comments_text, set_value=string(respiratory_signal->getcomments())

    ; redraw signal
    self->redraw

END


;;
;;
PRO CW_RESPIRATORY_SIGNAL::LOAD_RESPIRATORY_SIGNAL

    ; show file picker dialog
    filename = dialog_pickfile(title='Load Respiratory Signal From File', filter=[ '*.csv;*.csv2', '*.dat', '*.daf', '*.log', '*.sav'], /read, /must_exist, /fix_filter)
    if (filename eq '') then begin
        message, 'No file name to load from was selected. Doing nothing!', /informational
        return
    endif

    ; load signal data from a file
    respiratory_signal = obj_new('respiratorysignal')
    respiratory_signal->import_anyfile, filename

    ; set signal
    self->set_respiratory_signal, respiratory_signal

    ; update file name display
    file_text = widget_info(self->get_ui_area(), find_by_uname='file_text')
    widget_control, file_text, set_value=file_basename(filename)

END


;;
;;
PRO CW_RESPIRATORY_SIGNAL::SAVE_RESPIRATORY_SIGNAL
    
    ; check for valid signal data
    if (~self->is_data_valid()) then begin
        message, 'Could not save respiratory signal, since none is loaded currently!', /informational
        return
    endif

    ; show file picker dialog
    filename = dialog_pickfile(title='Save Respiratory Signal To File', filter=['*.sav'], /write, /overwrite_prompt)
    if (filename eq '') then begin
        message, 'No file name to save to was selected. Doing nothing!', /informational
        return
    endif

    ; save signal data to file
    (self.respiratory_signal)->save_savfile, filename

    ; update file name display
    file_text = widget_info(self->get_ui_area(), find_by_uname='file_text')
    widget_control, file_text, set_value=file_basename(filename)

END


;;
;;
PRO CW_RESPIRATORY_SIGNAL::INVERT_RESPIRATORY_SIGNAL

    ; check for valid signal data
    if (self->is_data_valid()) then begin

        ; invert signal
        signal = -((self.respiratory_signal)->getsignal())
        sampling_time = (self.respiratory_signal)->getsamplingtime()
        sensor = (self.respiratory_signal)->getsensor()
        comments = (self.respiratory_signal)->getcomments()
        (self.respiratory_signal)->setsignal, signal, sampling_time, sensor, comments

        ; reset draw geometry
        self->set_sampling_time, (self.respiratory_signal)->getsamplingtime()
        xsize = (self.respiratory_signal)->getsize()
        min = min((self.respiratory_signal)->getsignal(), max=max, /nan)
        ysize = max - min
        self->set_data_size, xsize, ysize
        widget_control, self->get_draw_area(), set_draw_view=[0,0]

    endif

END


;; TODO implement shifting by x and y
;;
PRO CW_RESPIRATORY_SIGNAL::PLOT_DATA, X, Y, XSIZE, YSIZE

    ; retrieve signal and draw it
    xsize <= (self->get_draw_size())[0]
    ysize <= (self->get_draw_size())[1]
    signal = self->get_signal()
    ymin = min(signal) + y/self.yscale + self.yoffset
    ymax = ymin + ysize/self.yscale
    signal_x = (findgen(xsize/self.xscale) + x/self.xscale) * self.sampling_time + self.xoffset
    signal_y = signal[(x/self.xscale):((x+xsize-1)/self.xscale)] + self.yoffset
    ;plot, signal_x, signal_y, position=[0.0, 0.0, 1.0, 1.0], /noclip, xstyle=1, ystyle=1, xticklen=0.1, yticklen=0.1
    plot, signal_x, signal_y, yrange=[ymin,ymax], xstyle=1, ystyle=1, xticklen=0.1, yticklen=0.1

END


;;
;;
FUNCTION CW_RESPIRATORY_SIGNAL::EVENT, EV

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='load_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->load_respiratory_signal
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='save_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->save_respiratory_signal
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='invert_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            self->invert_respiratory_signal
            return, 0
        endif
    endif

    if (ev.id eq widget_info(self->get_widget(), find_by_uname='syncronize_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            return, self->post_event(ev.top, 1)
        endif
    endif

    return, self->cw_advanced_draw_manipulator::event(ev)

END
