;; 
;;
PRO DIGITAL_FILTER__DEFINE

    ; define the class structure 
    struct = { DIGITAL_FILTER, INHERITS signal_filter, lower_frequency:0.0, upper_frequency:0.0, filter_power:0, terms:0 }

END


;; Creates new Respiratory Motion Filter.
;;
FUNCTION DIGITAL_FILTER::INIT, SAMPLING_TIME=SAMPLING_TIME, MULTIPLEX_DIMENSION=MULTIPLEX_DIMENSION, LOWER_FREQUENCY=LOWER_FREQUENCY, UPPER_FREQUENCY=UPPER_FREQUENCY, FILTER_POWER=FILTER_POWER, TERMS=TERMS

    ; initialize superclass
    if (self->signal_filter::init(sampling_time=sampling_time, multiplex_dimension=multiplex_dimension) eq 0) then begin
        return, 0
    endif

    ; try to initialize field values
    catch, e
    if (e ne 0) then begin

        message, !error_state.msg, /informational
        return, 0

    endif

        ; try to initialize field values
        self->set_lower_frequency, lower_frequency
        self->set_upper_frequency, upper_frequency
        self->set_filter_power, filter_power
        self->set_terms, terms

    catch, /cancel
    return, 1

END


;; 
;;
PRO DIGITAL_FILTER::CLEANUP

    self->signal_filter::cleanup

END


;; TODO check for a numeric value between 0.0 and 1.0
;;
PRO DIGITAL_FILTER::SET_LOWER_FREQUENCY, FREQ

    ; reset to 0.0, if no value given
    if (n_elements(freq) eq 0) then begin
        self.lower_frequency = 0.0
        return
    endif

    self.lower_frequency = freq

END


;;
;;
FUNCTION DIGITAL_FILTER::GET_LOWER_FREQUENCY

    return, self.lower_frequency

END


;; TODO check for a numeric value between 0.0 and 1.0
;;
PRO DIGITAL_FILTER::SET_UPPER_FREQUENCY, FREQ

    ; reset to 1.0, if no value given
    if (n_elements(freq) eq 0) then begin
        self.upper_frequency = 1.0
        return
    endif

    self.upper_frequency = freq

END


;;
;;
FUNCTION DIGITAL_FILTER::GET_UPPER_FREQUENCY

    return, self.upper_frequency

END


;; TODO check for a positive numeric value
;;
PRO DIGITAL_FILTER::SET_FILTER_POWER, POWER

    ; reset to 50, if no value given
    if (n_elements(power) eq 0) then begin
        self.filter_power = 50
        return
    endif

    self.filter_power = power

END


;;
;;
FUNCTION DIGITAL_FILTER::GET_FILTER_POWER

    return, self.filter_power

END


;; TODO check for a positive integer value
;;
PRO DIGITAL_FILTER::SET_TERMS, TERMS

    ; reset to 100, if no value given
    if (n_elements(terms) eq 0) then begin
        self.terms = 100
        return
    endif

    self.terms = terms

END


;;
;;
FUNCTION DIGITAL_FILTER::GET_TERMS

    return, self.terms

END


;;
;; TODO make scale correction optional, controlled by a keyword
;;
FUNCTION DIGITAL_FILTER::APPLY, DATA

    nyquist = self->get_nyquist_frequency()
    lower = self->get_lower_frequency()
    upper = self->get_upper_frequency()
    power = self->get_filter_power()
    terms = self->get_terms()

    filtered_data = convol(reform(data), digital_filter(lower/nyquist, upper/nyquist, power, terms, /double), /edge_truncate)
    scale_factor = total(data)/total(filtered_data)
    return, filtered_data * scale_factor

END
