;; 
;;
PRO RESPIRATORY_SIGNAL_GATED_CORRESPONDENCE_COMPARATOR__DEFINE

    ; define the class structure 
    struct = { RESPIRATORY_SIGNAL_GATED_CORRESPONDENCE_COMPARATOR, INHERITS abstract_respiratory_signal_gated_comparator }

END


;; 
;;
FUNCTION RESPIRATORY_SIGNAL_GATED_CORRESPONDENCE_COMPARATOR::INIT, DURATION=DURATION, GATING_METHOD=GATING_METHOD, NUMBER_OF_GATES=NUMBER_OF_GATES

    if (~self->abstract_respiratory_signal_gated_comparator::init(duration=duration, gating_method=gating_method, number_of_gates=number_of_gates)) then begin
        return, 0
    endif

    return, 1

END



;;
;;
PRO RESPIRATORY_SIGNAL_GATED_CORRESPONDENCE_COMPARATOR::CLEANUP

    self->abstract_respiratory_signal_gated_comparator::cleanup

END


;; TODO treat throw-away gates as missing data
;;
FUNCTION RESPIRATORY_SIGNAL_GATED_CORRESPONDENCE_COMPARATOR::COMPARE, RESPIRATORY_SIGNALS, OFFSETS

    ; get all gated signals
    number_of_gates = self->get_number_of_gates()
    gates_arrays = self->gate(respiratory_signals, offsets)
    gates_arrays_dimensions = size(gates_arrays, /dimensions)
    n_signals = gates_arrays_dimensions[0]
    n_samples = gates_arrays_dimensions[1]

    ; compute a matrix of arrays containing the absolute signum of the correspondences between the gated signals
    gates_correspondences_arrays = rebin([0], n_signals, n_signals, n_samples)
    valid_gates_samples = rebin([n_samples], n_signals, n_signals)
    for i = 0, n_signals-1 do begin
        for j = 0, n_signals-1 do begin
            ; compute absolute signum of the correspondence for the gated signal
            gates_correspondences_arrays[i,j,*] = gates_arrays[i,*] ne gates_arrays[j,*]
            ; compensate for invalid gates
            invalid_gates_indices = [where(gates_arrays[i,*] ge number_of_gates), where(gates_arrays[j,*] ge number_of_gates)]
            invalid_gates_indices = invalid_gates_indices[uniq(invalid_gates_indices, sort(invalid_gates_indices))]
            if (~array_equal(invalid_gates_indices, -1)) then begin 
                gates_correspondences_arrays[i,j,invalid_gates_indices] = 0
                valid_gates_samples[i,j] -= n_elements(invalid_gates_indices)
            endif
        endfor
    endfor

    ; compute a matrix of correspondence ratios between the gated signals
    gates_correspondences = 1 - (total(gates_correspondences_arrays, 3) / valid_gates_samples)
    return, gates_correspondences

END
