;; 
;;
PRO SCALE_FILTER__DEFINE

    ; define the class structure 
    struct = { SCALE_FILTER, INHERITS signal_filter, lower_boundary:0.0, upper_boundary:0.0 }

END


;; Creates new Scale Filter.
;;
FUNCTION SCALE_FILTER::INIT, MULTIPLEX_DIMENSION=MULTIPLEX_DIMENSION, LOWER_BOUNDARY=LOWER_BOUNDARY, UPPER_BOUNDARY=UPPER_BOUNDARY

    ; initialize superclass
    if (self->signal_filter::init(sampling_time=1.0, multiplex_dimension=multiplex_dimension) eq 0) then begin
        return, 0
    endif

    ; try to initialize field values
    catch, e
    if (e ne 0) then begin

        message, !error_state.msg, /informational
        return, 0

    endif

        ; try to initialize field values
        self->set_lower_boundary, lower_boundary
        self->set_upper_boundary, upper_boundary

    catch, /cancel
    return, 1

END


;; 
;;
PRO SCALE_FILTER::CLEANUP

    self->signal_filter::cleanup

END


;; TODO check for a numeric value
;;
PRO SCALE_FILTER::SET_LOWER_BOUNDARY, BOUNDARY

    ; reset to 0.0, if no value given
    if (n_elements(boundary) eq 0) then begin
        self.lower_boundary = 0.0
        return
    endif

    self.lower_boundary = boundary

END


;;
;;
FUNCTION SCALE_FILTER::GET_LOWER_BOUNDARY

    return, self.lower_boundary

END


;; TODO check for a numeric value
;;
PRO SCALE_FILTER::SET_UPPER_BOUNDARY, BOUNDARY

    ; reset to 1.0, if no value given
    if (n_elements(boundary) eq 0) then begin
        self.upper_boundary = 1.0
        return
    endif

    self.upper_boundary = boundary

END


;;
;;
FUNCTION SCALE_FILTER::GET_UPPER_BOUNDARY

    return, self.upper_boundary

END


;;
;;
FUNCTION SCALE_FILTER::APPLY, DATA

    ; calculate current min and max of the data
    min = min(data, max=max)

    ; calculate scale factor and offset
    scale_factor = (self.upper_boundary - self.lower_boundary) / (max - min)
    offset = self.lower_boundary - (min * scale_factor)

    ; transform data array to fit new boundaries
    return, offset + (scale_factor * data)

END
