;;
;;
PRO BASE_EVENT, EV

    if (ev.id eq widget_info(ev.top, find_by_uname='base')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_base', /fold_case)) then begin
            base_event_resize, ev.id, ev.x, ev.y
            return
        end
    endif

    respiratory_signal_area = widget_info(ev.top, find_by_uname='respiratory_signal_area')
    respiratory_signal_widgets = get_child_widgets(respiratory_signal_area)
    index = where(respiratory_signal_widgets eq ev.id, count)
    if (count ne 0) then begin
        if (strcmp(tag_names(ev, /structure), 'draw_manipulator_event', /fold_case)) then begin
            if (ev.type eq 1) then begin
                base_event_syncronize, ev.id, respiratory_signal_widgets
                return
            end
        end
    endif

    if (ev.id eq widget_info(ev.top, find_by_uname='comparator_button')) then begin
        if (strcmp(tag_names(ev, /structure), 'widget_button', /fold_case)) then begin
            base_event_compare, ev.top, respiratory_signal_widgets
            return
        end
    endif

END


;;
;;
PRO BASE_EVENT_RESIZE, TLB, X, Y

    ; get respiratory signal widgets
    respiratory_signal_area = widget_info(tlb, find_by_uname='respiratory_signal_area')
    respiratory_signal_widgets = get_child_widgets(respiratory_signal_area)

    ; get number of respiratory signal widgets
    if (array_equal(respiratory_signal_widgets, -1)) then begin
        n_respiratory_signal_widgets = 0
    endif else begin
        n_respiratory_signal_widgets = n_elements(respiratory_signal_widgets)
    endelse

    ; TODO fix size calculation! why is a correction of 30 pixels needed?
    tlb_geometry = widget_info(tlb, /geometry)
    xsize = (x - 2*tlb_geometry.xpad - 60)
    ysize = (2.4*y - 2*tlb_geometry.ypad - n_respiratory_signal_widgets*tlb_geometry.space) / n_respiratory_signal_widgets

    widget_control, tlb, xsize=xsize, ysize=ysize * n_respiratory_signal_widgets
    widget_control, tlb, scr_xsize=x, scr_ysize=y

    for i = 0, n_respiratory_signal_widgets-1 do begin
        widget_control, respiratory_signal_widgets[i], send_event={widget_base, id:respiratory_signal_widgets[i], top:tlb, handler:respiratory_signal_widgets[i], x:xsize, y:ysize}, /no_copy
    endfor

END


;;
;;
PRO BASE_EVENT_SYNCRONIZE, MASTER, SLAVES

    ; get the master sampling time to syncronize to
    widget_control, master, get_value=draw_manipulator
    master_sampling_time = draw_manipulator->get_sampling_time() / draw_manipulator->get_xscale()

    ; set the scale of all slaves to match the masters sampling time
    for i = 0, n_elements(slaves)-1 do begin
        widget_control, slaves[i], get_value=draw_manipulator
        draw_manipulator->set_xscale, draw_manipulator->get_sampling_time() / master_sampling_time
    endfor

END


;;
;;
PRO BASE_EVENT_COMPARE, TLB, RESPIRATORY_SIGNAL_WIDGETS

    ; create the desired comparator
    comparator_id = widget_info(widget_info(tlb, find_by_unam='comparator_droplist'), /droplist_select)
    if (comparator_id eq 0) then begin
        comparator = obj_new('respiratory_signal_correlation_comparator')
    endif else if (comparator_id eq 1) then begin
        comparator = obj_new('respiratory_signal_gated_correlation_comparator')
    endif else if (comparator_id eq 2) then begin
        comparator = obj_new('respiratory_signal_gated_correspondence_comparator')
    endif else begin
        message, 'An unsupported comparator was selected. This should never have happened!'
    endelse

    ; setup gating parameters
    if (obj_isa(comparator, 'abstract_respiratory_signal_gated_comparator')) then begin
        widget_control, widget_info(tlb, find_by_unam='comparator_gating_method_text'), get_value=gating_method
        comparator->set_gating_method, gating_method
        widget_control, widget_info(tlb, find_by_unam='comparator_number_of_gates_text'), get_value=number_of_gates
        comparator->set_number_of_gates, number_of_gates
    endif

    ; setup duration of comparison
    widget_control, widget_info(tlb, find_by_unam='comparator_duration_text'), get_value=comparator_duration
    comparator->set_duration, comparator_duration

    ; get all respiratory signals
    n_signals = n_elements(respiratory_signal_widgets)
    respiratory_signals = objarr(n_signals)
    offsets = fltarr(n_signals)
    for i = 0, n_signals-1 do begin
        widget_control, respiratory_signal_widgets[i], get_value=respiratory_signal_widget
        if (respiratory_signal_widget->is_data_valid()) then begin
            respiratory_signals[i] = respiratory_signal_widget->get_respiratory_signal()
            offsets[i] = -(respiratory_signal_widget->get_xoffset())
        end
    end

    ; discard invalid respiratory signals
    valid_indices = where(obj_valid(respiratory_signals))
    if (n_elements(valid_indices) le 1) then begin
        message, 'At least two respiratory signals must be loaded in order to perform a comparison!', /informational
        obj_destroy, comparator
        return
    end
    respiratory_signals = respiratory_signals[valid_indices]
    offsets = offsets[valid_indices]

    ; apply comparator
    comparison_matrix = rebin([!values.f_nan], n_signals, n_signals)
    valid_matrix_indices = ((indgen(n_signals, n_signals))[valid_indices,*])[*,valid_indices]
    comparison_matrix[valid_matrix_indices] = comparator->compare(respiratory_signals, offsets)
    obj_destroy, comparator

    ; output the results of the comparison in a new modal window
    comparison_table = obj_new('cw_comparison_table', comparison_matrix, group_leader=tlb, /modal)

END


;;
;;
FUNCTION GET_CHILD_WIDGETS, PARENT

    ; are there any child widgets at all?
    first_child = widget_info(parent, /child)
    if (first_child eq 0) then begin
        return, -1
    endif

    ; create list of all child widgets
    children = [first_child]
    current_child = widget_info(first_child, /sibling)
    while (current_child ne 0) do begin
        children = [children, current_child]
        current_child = widget_info(current_child, /sibling)
    endwhile
    return, children

END


;;
;;
PRO SIGNAL_ANALYSIS_GUI

    ; setup widget hierarchy
    base = widget_base(uname='base', title='Respiratory Signal Analysis', /column, /scroll, /tlb_size_events)
    respiratory_signal_area = widget_base(base, uname='respiratory_signal_area', /column)

    ; setup respiratory signal widgets
    respiratory_signal_widget = obj_new('cw_marker_based_respiratory_signal', respiratory_signal_area, uname='respiratory_signal', frame=1)
    respiratory_signal_widget = obj_new('cw_marker_based_respiratory_signal', respiratory_signal_area, uname='respiratory_signal', frame=1)
    respiratory_signal_widget = obj_new('cw_marker_based_respiratory_signal', respiratory_signal_area, uname='respiratory_signal', frame=1)
    respiratory_signal_widget = obj_new('cw_marker_based_respiratory_signal', respiratory_signal_area, uname='respiratory_signal', frame=1)

    ; setup comparator form
    comparator_form = widget_base(base, uname='comparator_form', /row, frame=1)
    comparator_droplist = widget_droplist(comparator_form, uname='comparator_droplist', value=['Correlation Comparator', 'Gated Correlation Comparator', 'Gated Correspondence Comparator'])
    comparator_gating_method_label = widget_label(comparator_form, value='Gating Method:')
    comparator_gating_method_text = widget_text(comparator_form, uname='comparator_gating_method_text', value='1', xsize=2, /editable)
    comparator_number_of_gates_label = widget_label(comparator_form, value='Number of Gates:')
    comparator_number_of_gates_text = widget_text(comparator_form, uname='comparator_number_of_gates_text', value='4', xsize=3, /editable)
    comparator_button = widget_button(comparator_form, uname='comparator_button', value='Compare All Signals...')
    comparator_duration_label = widget_label(comparator_form, value='Duration (s):')
    comparator_duration_text = widget_text(comparator_form, uname='comparator_duration_text', value='-1', xsize=6, /editable)
    
    ; realize widget and set initial size
    widget_control, base, /realize
    base_geometry = widget_info(base, /geometry)
    size = get_screen_size() * 0.9
    base_event_resize, base, size[0], size[1]
    xmanager, 'base', base

END
