;; 
;;
PRO SIGNAL_FILTER__DEFINE

    ; define the class structure 
    struct = { SIGNAL_FILTER, sampling_time:0.0, multiplex_dimension:0 }

END


;; Creates new Signal Filter.
;;
FUNCTION SIGNAL_FILTER::INIT, SAMPLING_TIME=SAMPLING_TIME, MULTIPLEX_DIMENSION=MULTIPLEX_DIMENSION

    ; try to setup fields
    catch, e
    if (e ne 0) then begin

        message, !error_state.msg, /informational
        return, 0

    endif

        ; try to setup fields
        self->set_sampling_time, sampling_time
        self->set_multiplex_dimension, multiplex_dimension

    catch, /cancel
    return, 1

END


;; 
;;
PRO SIGNAL_FILTER::CLEANUP
END


;; TODO check for a positive numeric value
;;
PRO SIGNAL_FILTER::SET_SAMPLING_TIME, TIME

    ; reset to 1.0, if no value given
    if (n_elements(time) eq 0) then begin
        self.sampling_time = 1.0
        return
    endif

    self.sampling_time = time

END


;;
;;
PRO SIGNAL_FILTER::SET_NYQUIST_FREQUENCY, TIME

    self.sampling_time = 0.5 / time

END


;;
;;
FUNCTION SIGNAL_FILTER::GET_SAMPLING_TIME

    return, self.sampling_time

END


;;
;;
FUNCTION SIGNAL_FILTER::GET_NYQUIST_FREQUENCY

    return, 0.5 / self.sampling_time

END


;; TODO check for a valid dimension index
;; 0 means over all dimension lineary
;;
PRO SIGNAL_FILTER::SET_MULTIPLEX_DIMENSION, DIMENSION

    ; reset to 0, if no value given
    if (n_elements(dimension) eq 0) then begin
        self.multiplex_dimension = 0
        return
    endif

    self.multiplex_dimension = dimension
END


;;
;;
FUNCTION SIGNAL_FILTER::GET_MULTIPLEX_DIMENSION

    return, self.multiplex_dimension

END


;; A template method.
;;
FUNCTION SIGNAL_FILTER::APPLY, DATA

    return, data

END


;; 
;;
FUNCTION SIGNAL_FILTER::MULTIPLEX_APPLY, DATA

    n_data = n_elements(data)
    n_dimensions = size(data, /n_dimensions)
    dimensions = size(data, /dimensions)
    multiplex_dimension = self->get_multiplex_dimension()

    ; check array data
    if (n_data eq 0) then begin
        message, 'No data to filter given!'
    endif

    ; apply filter over all dimensions serially?
    if (multiplex_dimension le 0) then begin
        return, self->apply(reform(data, n_data))
    endif 

    ; check if array has a sufficient number of dimensions
    if (n_dimensions lt multiplex_dimension) then begin
        message, 'Data must have at least ' + int2str(multiplex_dimension) + ' dimensions, since this signal filter is setup to work over that dimension.'
    endif

    ; initialize array to hold the filter results
    filtered_data = data

    ; setup helper variables for indexing the correct signal data
    dimension_offsets = [1ul, dimensions]
    for i = 0, n_dimensions-1 do begin
        dimension_offsets[i+1] *= dimension_offsets[i]
    endfor
    signal_offsets = ulindgen(dimensions[multiplex_dimension-1]) * dimension_offsets[multiplex_dimension-1]

    ; iterate over all signal data in the specified dimension 
    signal_start_index = rebin([0ul], n_dimensions)
    repeat begin

        ; get indices of current signal data
        signal_indices = signal_offsets + total(signal_start_index * dimension_offsets)
        filtered_data[signal_indices] = self->apply(data[signal_indices])
        
        ; update signal start index by increasing the first dimension and overflowing to other dimensions as necessary
        ; the dimension containing the signal data is skipped
        for i = 0ul, n_dimensions-1 do begin
            if (i eq multiplex_dimension-1) then begin
                continue
            endif else if (++signal_start_index[i] lt dimensions[i]) then begin
                break
            endif else begin
                signal_start_index[i] = 0
            endelse
        endfor

    endrep until total(signal_start_index) eq 0

    return, filtered_data

END
